
cdef extern from "primetester.h":
     long next_prime(long prime)

cdef extern from "mmapbitarray.h":
     ctypedef struct MBArray:
         long bits
         long size
         long bytes
         long preamblebytes
         char * filename
         int fd

     MBArray * mbarray_ClearAll(MBArray * array)
     MBArray * mbarray_Sync(MBArray * array)
     MBArray * mbarray_And(MBArray * dest, MBArray * src)
     MBArray * mbarray_Or(MBArray * dest, MBArray * src)
     MBArray * mbarray_Xor(MBArray * dest, MBArray * src)
     MBArray * mbarray_And_Ternary(MBArray * dest, MBArray * a, MBArray * b)
     MBArray * mbarray_Or_Ternary(MBArray * dest, MBArray * a, MBArray * b)
     MBArray * mbarray_Xor_Ternary(MBArray * dest, MBArray * a, MBArray * b)
     int mbarray_Update(MBArray * array, char * data, int size)
     int mbarray_FileSize(MBArray * array)
     char * mbarray_CharData(MBArray * array)


cdef extern from "bloomfilter.h":
     ctypedef struct BloomFilter:
         long max_num_elem
         double error_rate
         int num_hashes
         long * hash_seeds
         MBArray * array
         unsigned char bf_version
         unsigned char count_correct
         unsigned long long elem_count

     ctypedef struct Key:
         long nhash
         char * shash

     BloomFilter * bloomfilter_Create_Mmap(long max_num_elem,
                                      double error_rate,
                                      char * fname, long num_bits,
                                      int oflags, int perms,
                                      int * hash_seeds, int num_hashes)
     BloomFilter * bloomfilter_Create_Malloc(long max_num_elem,
                                      double error_rate,
                                      long num_bits,
                                      int * hash_seeds, int num_hashes)
     void bloomfilter_Destroy(BloomFilter * bf)
     int bloomfilter_Add(BloomFilter * bf, Key * key)
     int bloomfilter_Test(BloomFilter * bf, Key * key)
     int bloomfilter_Update(BloomFilter * bf, char * data, int size)
     BloomFilter * bloomfilter_Copy_Template(BloomFilter * src, char * filename, int perms)



# cython: language_level=3

VERSION = (0, 5, 2)
AUTHOR = "Michael Axiak"

__VERSION__ = VERSION


cimport cbloomfilter
cimport cpython

import array
from base64 import b64encode, b64decode
import errno as eno
import math
import os
import random
import shutil
import sys
import warnings
import zlib


cdef extern int errno
cdef NoConstruct = object()


cdef _construct_access(mode):
    result = os.F_OK
    if 'w' in mode:
        result |= os.W_OK
    if 'r' in mode:
        result |= os.R_OK
    return result


cdef _construct_mode(mode):
    result = os.O_RDONLY
    if 'w' in mode:
        result |= os.O_RDWR
    if 'b' in mode and hasattr(os, 'O_BINARY'):
        result |= os.O_BINARY
    if mode.endswith('+'):
        result |= os.O_CREAT
    return result


class IndeterminateCountError(ValueError):
    pass


cdef class BloomFilter:
    """
    Creates a new BloomFilter object with a given capacity and error_rate.

    :param int capacity: the maximum number of elements this filter
        can contain while keeping the false positive rate under ``error_rate``.
    :param float error_rate: false positive probability that will hold
        given that ``capacity`` is not exceeded.
    :param str filename: filename to use to create the new Bloom filter.
        If a filename is not provided, an in-memory Bloom filter will be created.
    :param str mode: (*not applicable for an in-memory Bloom filter*)
        file access mode.
    :param int perm: (*not applicable for an in-memory Bloom filter*)
        file access permission flags.
    :param list hash_seeds: optionally specify hash seeds to use for the
        hashing algorithm. Each hash seed must not exceed 32 bits. The number
        of hash seeds will determine the number of hashes performed.

    **Note that we do not check capacity.** This is important, because
    we want to be able to support logical OR and AND (see :meth:`BloomFilter.union`
    and :meth:`BloomFilter.intersection`). The capacity and error_rate then together
    serve as a contract -- you add less than capacity items, and the Bloom filter
    will have an error rate less than error_rate.

    Raises :class:`OSError` if the supplied filename does not exist or if user
    lacks permission to access such file. Raises :class:`ValueError` if the supplied
    ``error_rate`` is invalid, ``hash_seeds`` does not contain valid hash seeds, or
    if the file cannot be read.
    """

    cdef cbloomfilter.BloomFilter * _bf
    cdef int _closed
    cdef int _in_memory
    cdef int _oflags

    def __cinit__(self, capacity, error_rate, filename=None, perm=0755, hash_seeds=None):
        self._closed = 0
        self._in_memory = 0
        self._oflags = os.O_RDWR

        if capacity is NoConstruct:
            return

        self._create(capacity, error_rate, filename, perm, hash_seeds)

    def _create(self, capacity, error_rate, filename=None, perm=0755, hash_seeds=None):
        cdef char * seeds
        cdef long long num_bits

        # Make sure that if the filename is defined, that the
        # file exists
        if filename and os.path.exists(filename):
            os.unlink(filename)

        # For why we round down for determining the number of hashes:
        # http://corte.si/%2Fposts/code/bloom-filter-rules-of-thumb/index.html
        # "The number of hashes determines the number of bits that need to
        # be read to test for membership, the number of bits that need to be
        # written to add an element, and the amount of computation needed to
        # calculate hashes themselves. We may sometimes choose to use a less
        # than optimal number of hashes for performance reasons (especially
        # when we choose to round down when the calculated optimal number of
        # hashes is fractional)."

        if not (0 < error_rate < 1):
            raise ValueError("error_rate allowable range (0.0, 1.0) %f" % (error_rate,))

        array_seeds = array.array('I')

        if hash_seeds:
            for seed in hash_seeds:
                if not isinstance(seed, int) or seed < 0 or seed.bit_length() > 32:
                    raise ValueError("invalid hash seed '%s', must be >= 0 "
                                        "and up to 32 bits in size" % seed)
            num_hashes = len(hash_seeds)
            array_seeds.extend(hash_seeds)
        else:
            num_hashes = max(math.floor(math.log2(1 / error_rate)), 1)
            array_seeds.extend([random.getrandbits(32) for i in range(num_hashes)])

        test = array_seeds.tobytes()
        seeds = test

        bits_per_hash = math.ceil(
                capacity * abs(math.log(error_rate)) /
                (num_hashes * (math.log(2) ** 2)))

        # Minimum bit vector of 128 bits
        num_bits = max(num_hashes * bits_per_hash,128)

        # print("k = %d  m = %d  n = %d   p ~= %.8f" % (
        #     num_hashes, num_bits, capacity,
        #     (1.0 - math.exp(- float(num_hashes) * float(capacity) / num_bits))
        #     ** num_hashes))

        # If a filename is provided, we should make a mmap-file
        # backed bloom filter. Otherwise, it will be malloc
        if filename:
            self._bf = cbloomfilter.bloomfilter_Create_Mmap(capacity,
                                                    error_rate,
                                                    filename.encode(),
                                                    num_bits,
                                                    self._oflags | os.O_CREAT,
                                                    perm,
                                                    <int *>seeds,
                                                    num_hashes)
        else:
            self._in_memory = 1
            self._bf = cbloomfilter.bloomfilter_Create_Malloc(capacity,
                                                    error_rate,
                                                    num_bits,
                                                    <int *>seeds,
                                                    num_hashes)
        if self._bf is NULL:
            if filename:
                raise OSError(errno, '%s: %s' % (os.strerror(errno),
                                                    filename))
            else:
                cpython.PyErr_NoMemory()

    def _open(self, filename, mode="rw"):
        # Should not overwrite
        mode = mode.replace("+", "")

        if not os.path.exists(filename):
            raise OSError(eno.ENOENT, '%s: %s' % (os.strerror(eno.ENOENT),
                                                        filename))
        if not os.access(filename, _construct_access(mode)):
            raise OSError("Insufficient permissions for file %s" % filename)

        self._oflags = _construct_mode(mode)
        self._bf = cbloomfilter.bloomfilter_Create_Mmap(0,
                                                0,
                                                filename.encode(),
                                                0,
                                                self._oflags,
                                                0,
                                                NULL, 0)
        if self._bf is NULL:
            raise ValueError("Invalid %s file: %s" %
                                (self.__class__.__name__, filename))

    def __dealloc__(self):
        cbloomfilter.bloomfilter_Destroy(self._bf)
        self._bf = NULL

    @property
    def bit_array(self):
        """Bit vector representation of the Bloom filter contents.
        Returns an integer.
        """
        self._assert_open()
        start_pos = self._bf.array.preamblebytes
        end_pos = start_pos + self._bf.array.bytes
        arr = (<char *>cbloomfilter.mbarray_CharData(self._bf.array))[start_pos:end_pos]
        return int.from_bytes(arr, byteorder="big", signed=False)

    @property
    def hash_seeds(self):
        """Integer seeds used for the random hashing. Returns a list of integers."""
        self._assert_open()
        seeds = array.array('I')
        seeds.frombytes(
            (<char *>self._bf.hash_seeds)[:4 * self.num_hashes]
        )
        return seeds

    @property
    def capacity(self):
        """The maximum number of elements this filter can contain while keeping
        the false positive rate under :attr:`BloomFilter.error_rate`.
        Returns an integer.
        """
        self._assert_open()
        return self._bf.max_num_elem

    @property
    def error_rate(self):
        """The acceptable probability of false positives. Returns a float."""
        self._assert_open()
        return self._bf.error_rate

    @property
    def num_hashes(self):
        """Number of hash functions used when computing."""
        self._assert_open()
        return self._bf.num_hashes

    @property
    def num_bits(self):
        """Number of bits used in the filter as buckets."""
        self._assert_open()
        return self._bf.array.bits

    def _name(self):
        self._assert_open()
        if self._in_memory:
            raise NotImplementedError('Cannot access .name on an in-memory %s'
                                        % self.__class__.__name__)
        if self._bf.array.filename is NULL:
            return None
        return self._bf.array.filename

    @property
    def name(self):
        """PENDING DEPRECATION - use :meth:`BloomFilter.filename` instead.

        File name (compatible with file objects). Does not apply to an in-memory
        :class:`BloomFilter` and will raise :class:`ValueError` if accessed.
        Returns an encoded string.
        """
        warnings.warn('name will be deprecated in future versions, use '
                      'filename instead', PendingDeprecationWarning)
        return self._name()

    @property
    def filename(self):
        """File name (compatible with file objects). Does not apply to an in-memory
        :class:`BloomFilter` and will raise :class:`ValueError` if accessed.
        Returns a string.
        """
        return self._name().decode()

    @property
    def read_only(self):
        """Indicates if the opened :class:`BloomFilter` is read-only.
        Always ``False`` for an in-memory :class:`BloomFilter`.
        """
        self._assert_open()
        return not self._in_memory and not self._oflags & os.O_RDWR

    def fileno(self):
        """Bloom filter file descriptor."""
        self._assert_open()
        return self._bf.array.fd

    def __repr__(self):
        self._assert_open()
        my_name = self.__class__.__name__
        return '<%s capacity: %d, error: %0.3f, num_hashes: %d>' % (
            my_name, self._bf.max_num_elem, self._bf.error_rate,
            self._bf.num_hashes)

    def __str__(self):
        return self.__repr__()

    def sync(self):
        """Forces a ``sync()`` call on the underlying mmap file object. Use this if
        you are about to copy the file and you want to be sure you got
        everything correctly.
        """
        self._assert_open()
        self._assert_writable()
        cbloomfilter.mbarray_Sync(self._bf.array)

    def clear_all(self):
        """Removes all elements from the Bloom filter at once."""
        self._assert_open()
        self._assert_writable()
        cbloomfilter.mbarray_ClearAll(self._bf.array)

    def __contains__(self, item_):
        """Checks to see if item is contained in the filter, with
        an acceptable false positive rate of :attr:`BloomFilter.error_rate`.

        :param item: hashable object
        :rtype: bool
        """
        self._assert_open()
        cdef cbloomfilter.Key key
        if isinstance(item_, str):
            item = item_.encode()
            key.shash = item
            key.nhash = len(item)
        else:
            item = item_
            key.shash = NULL
            key.nhash = hash(item)
        return cbloomfilter.bloomfilter_Test(self._bf, &key) == 1

    def copy_template(self, filename, perm=0755):
        """Creates a new :class:`BloomFilter` object with the exact same parameters.
        Once this is performed, the two filters are comparable, so
        you can perform set operations using logical operators.

        :param str filename: new filename
        :param int perm: file access permission flags
        :rtype: :class:`BloomFilter`
        """
        self._assert_open()
        cdef BloomFilter copy = BloomFilter(NoConstruct, 0)
        if os.path.exists(filename):
            os.unlink(filename)
        copy._bf = cbloomfilter.bloomfilter_Copy_Template(self._bf, filename.encode(), perm)
        return copy

    def copy(self, filename):
        """Copies the current :class:`BloomFilter` object to another object
        with a new filename.

        :param str filename: new filename
        :rtype: :class:`BloomFilter`
        """
        self._assert_open()
        if self._in_memory:
            raise NotImplementedError('Cannot call .copy on an in-memory %s' %
                                      self.__class__.__name__)
        shutil.copy(self._bf.array.filename, filename)
        return self.open(filename)

    def add(self, item_):
        """Adds an item to the Bloom filter. Returns a boolean indicating whether
        this item was present in the Bloom filter prior to adding
        (see :meth:`BloomFilter.__contains__`).

        :param item: hashable object
        :rtype: bool
        """
        self._assert_open()
        self._assert_writable()
        cdef cbloomfilter.Key key
        if isinstance(item_, str):
            item = item_.encode()
            key.shash = item
            key.nhash = len(item)
        else:
            item = item_
            key.shash = NULL
            key.nhash = hash(item)

        result = cbloomfilter.bloomfilter_Add(self._bf, &key)
        if result == 2:
            raise RuntimeError("Some problem occured while trying to add key.")
        return bool(result)

    def update(self, iterable):
        """Calls :meth:`BloomFilter.add` on all items in the iterable."""
        for item in iterable:
            self.add(item)

    def __len__(self):
        """Returns the number of distinct elements that have been
        added to the :class:`BloomFilter` object, subject to the error
        given in :attr:`BloomFilter.error_rate`.

        Raises :class:`IndeterminateCountError` if a the Bloom filter
        was a result of a set operation.

        :param item: hashable object
        :rtype: int
        """
        self._assert_open()
        if not self._bf.count_correct:
            raise IndeterminateCountError("Length of %s object is unavailable "
                                          "after intersection or union called." %
                                          self.__class__.__name__)
        return self._bf.elem_count

    def close(self):
        """Closes the currently opened :class:`BloomFilter` file descriptor.
        Following accesses to this instance will raise a :class:`ValueError`.

        *Caution*: this will delete an in-memory filter irrecoverably!
        """
        if self._closed == 0:
            self._closed = 1
            cbloomfilter.bloomfilter_Destroy(self._bf)
            self._bf = NULL

    def union(self, BloomFilter other):
        """Performs a set OR with another comparable filter. You can (only) construct
        comparable filters with :meth:`BloomFilter.copy_template` above.

        The computation will occur *in place*. That is, calling::

            >>> bf.union(bf2)

        is a way of adding *all* the elements of ``bf2`` to ``bf``.

        *NB: Calling this function will render future calls to len()
        invalid.*

        :param BloomFilter other: filter to perform the union with
        :rtype: :class:`BloomFilter`
        """
        self._assert_open()
        self._assert_writable()
        other._assert_open()
        self._assert_comparable(other)
        cbloomfilter.mbarray_Or(self._bf.array, other._bf.array)
        self._bf.count_correct = 0
        return self

    def __ior__(self, BloomFilter other):
        """See :meth:`BloomFilter.union`."""
        return self.union(other)

    def intersection(self, BloomFilter other):
        """The same as :meth:`BloomFilter.union` above except it uses
        a set AND instead of a set OR.

        *NB: Calling this function will render future calls to len()
        invalid.*

        :param BloomFilter other: filter to perform the intersection with
        :rtype: :class:`BloomFilter`
        """
        self._assert_open()
        self._assert_writable()
        other._assert_open()
        self._assert_comparable(other)
        cbloomfilter.mbarray_And(self._bf.array, other._bf.array)
        self._bf.count_correct = 0
        return self

    def __iand__(self, BloomFilter other):
        """See :meth:`BloomFilter.intersection`."""
        return self.intersection(other)

    def _assert_open(self):
        if self._closed != 0:
            raise ValueError("I/O operation on closed file")

    def _assert_writable(self):
        if self.read_only:
            raise ValueError("Write operation on read-only file")

    def _assert_comparable(self, BloomFilter other):
        error = ValueError("The two %s objects are not the same type (hint: "
                           "use copy_template)" % self.__class__.__name__)
        if self._bf.array.bits != other._bf.array.bits:
            raise error
        if self.hash_seeds != other.hash_seeds:
            raise error
        return

    def to_base64(self):
        """Creates a compressed, base64 encoded version of the :class:`BloomFilter`.
        Since the Bloom filter is efficiently in binary on the file system,
        this may not be too useful. I find it useful for debugging so I can
        copy filters from one terminal to another in their entirety.

        :rtype: base64 encoded string representing filter
        """
        self._assert_open()
        bfile = open(self.filename, "rb")
        fl_content = bfile.read()
        result = b64encode(zlib.compress(b64encode(zlib.compress(
            fl_content, 9))))
        bfile.close()
        return result

    @classmethod
    def from_base64(cls, filename, string, perm=0755):
        """Unpacks the supplied base64 string (as returned by :meth:`BloomFilter.to_base64`)
        into the supplied filename and return a :class:`BloomFilter` object using that
        file.

        :param str filename: new filename
        :param int perm: file access permission flags
        :rtype: :class:`BloomFilter`
        """
        bfile_fp = os.open(filename, _construct_mode("w+"), perm)
        os.write(bfile_fp, zlib.decompress(b64decode(zlib.decompress(
            b64decode(string)))))
        os.close(bfile_fp)
        return cls.open(filename)

    @classmethod
    def open(cls, filename, mode="rw"):
        """Creates a :class:`BloomFilter` object from an existing file.

        :param str filename: existing filename
        :param str mode: file access mode
        :rtype: :class:`BloomFilter`
        """
        instance = cls(NoConstruct, 0)
        instance._open(filename, mode)
        return instance



#!/usr/bin/env python

import sys
import os
import tempfile
import pybloomfilter

WORDS_FILE = os.path.join(os.path.dirname(__file__), 'words')
TEST_WORDS = os.path.join(os.path.dirname(__file__), 'testwords')

def main():
    global pybloomfilter

    if len(sys.argv) > 1 and sys.argv[1].lower() == '-pybloom':
        import pybloom
        pybloomfilter = pybloom

    with open(WORDS_FILE) as base_file:
        with open(TEST_WORDS) as test_file:
            base_words = set(base_file)
            test_words = set(test_file)
            correct_overlap = len(base_words & test_words)
            num_test_words = len(test_words)
            number_words = len(base_words)

    for error_rate in (0.01, 0.001, 0.0001):
        test_errors(error_rate, number_words, correct_overlap, num_test_words)


def test_errors(error_rate, filter_size, correct_overlap, num_test_words):
    bloom_file = tempfile.NamedTemporaryFile()
    try:
        bf = pybloomfilter.BloomFilter(filter_size, error_rate, bloom_file.name)
    except TypeError:
        bf = pybloomfilter.BloomFilter(filter_size, error_rate)

    with open(WORDS_FILE) as source_file:
        with open(TEST_WORDS) as test_file:
            run_test(bf, source_file, test_file, correct_overlap, num_test_words, error_rate)

    #os.unlink(bloom_file.name)


def run_test(bf, source_file, test_file, correct_overlap, num_test_words, error_rate):
    for word in source_file:
        bf.add(word.rstrip())

    positive_matches = sum(1 for word in test_file
                           if word.rstrip() in bf)


    actual_error_rate = float(positive_matches - correct_overlap) / correct_overlap

    print("Specified: %f; Measured: %f; num_hashes: %d, num_bits: %d" % (
        error_rate,
        actual_error_rate,
        getattr(bf, 'num_hashes', None) or getattr(bf, 'num_slices'),
        bf.num_bits,
        ))



if __name__ == '__main__':
    main()



import os
import sys

from setuptools import setup, Extension

if sys.version_info < (3, 5):
    raise SystemError("This package is for Python 3.5 and above.")

here = os.path.dirname(__file__)

# Get the long description from the README file
with open(os.path.join(here, "README.markdown"), encoding="utf-8") as fp:
    long_description = fp.read()

setup_kwargs = {}

ext_files = [
    "src/mmapbitarray.c",
    "src/bloomfilter.c",
    "src/md5.c",
    "src/primetester.c",
    "src/MurmurHash3.c",
]

# Branch out based on `--cython` in `argv`. Specifying `--cython` will try to cythonize source whether
# Cython module is available or not (`force_cythonize`).
cythonize = True
force_cythonize = False

if "--cython" in sys.argv:
    force_cythonize = True
    sys.argv.remove("--cython")

# Always try to cythonize `pybloomfilter.pyx` if Cython is available
# or if `--cython` was passed
try:
    from Cython.Distutils import build_ext
except ModuleNotFoundError:
    if force_cythonize:
        print(
            "Cannot Cythonize: Cython module not found. "
            "Hint: to build pybloomfilter using the distributed "
            "source code, simply run 'python setup.py install'."
        )
        sys.exit(1)
    cythonize = False

if cythonize:
    ext_files.append("src/pybloomfilter.pyx")
    setup_kwargs["cmdclass"] = {"build_ext": build_ext}
else:
    # Use `pybloomfilter.c` distributed with the package.
    # Note that we let the exception bubble up if `pybloomfilter.c` doesn't exist.
    ext_files.append("src/pybloomfilter.c")

ext_modules = [Extension("pybloomfilter", ext_files)]

setup(
    name="pybloomfiltermmap3",
    version="0.5.2",
    author="Prashant Sinha",
    author_email="prashant@noop.pw",
    url="https://github.com/prashnts/pybloomfiltermmap3",
    description="A fast implementation of Bloom filter for Python 3 built on mmap",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT License",
    test_suite="tests.test_all",
    ext_modules=ext_modules,
    python_requires=">=3.5, <4",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: C",
        "Programming Language :: Cython",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    **setup_kwargs
)



import os
import string
import unittest
import tempfile
from random import randint, choice, getrandbits

import pybloomfilter

from tests import with_test_file


class SimpleTestCase(unittest.TestCase):
    FILTER_SIZE = 200
    FILTER_ERROR_RATE = 0.001

    def setUp(self):
        # Convenience file-backed bloomfilter
        self.tempfile = tempfile.NamedTemporaryFile(suffix='.bloom',
                                                    delete=False)
        self.bf = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                            self.FILTER_ERROR_RATE,
                                            self.tempfile.name)

        # Convenience memory-backed bloomfilter
        self.bf_mem = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                                self.FILTER_ERROR_RATE)

    def tearDown(self):
        os.unlink(self.tempfile.name)

    def assertPropertiesPreserved(self, old_bf, new_bf):
        # Assert that a "new" BloomFilter has the same properties as an "old"
        # one.
        failures = []
        for prop in ['capacity', 'error_rate', 'num_hashes', 'num_bits',
                     'hash_seeds']:
            old, new = getattr(old_bf, prop), getattr(new_bf, prop)
            if new != old:
                failures.append((prop, old, new))
        self.assertEqual([], failures)

    def _random_str(self, length=16):
        chars = string.ascii_letters
        return ''.join(choice(chars) for _ in range(length))

    def _random_set_of_stuff(self, c):
        """
        Return a random set containing up to "c" count of each type of Python
        object.
        """
        return set(
            # Due to a small chance of collision, there's no guarantee on the
            # count of elements in this set, but we'll make sure that's okay.
            [self._random_str() for _ in range(c)] +
            [randint(-1000, 1000) for _ in range(c)] +
            [(randint(-200, 200), self._random_str()) for _ in range(c)] +
            [float(randint(10, 100)) / randint(10, 100)
             for _ in range(c)] +
            [int(randint(50000, 1000000)) for _ in range(c)] +
            [object() for _ in range(c)] +
            [str(self._random_str) for _ in range(c)])

    def _populate_filter(self, bf, use_update=False):
        """
        Populate given BloomFilter with a handfull of hashable things.
        """
        self._in_filter = self._random_set_of_stuff(10)
        self._not_in_filter = self._random_set_of_stuff(15)
        # Just in case we randomly chose a key which was also in
        # self._in_filter...
        self._not_in_filter = self._not_in_filter - self._in_filter

        if use_update:
            bf.update(self._in_filter)
        else:
            for item in self._in_filter:
                bf.add(item)

    def _check_filter_contents(self, bf):
        for item in self._in_filter:
            # We should *never* say "not in" for something which was added
            self.assertTrue(item in bf, '%r was NOT in %r' % (item, bf))

        # We might say something is in the filter which isn't; we're only
        # trying to test correctness, here, so we are very lenient.  If the
        # false positive rate is within 2 orders of magnitude, we're okay.
        false_pos = len(list(filter(bf.__contains__, self._not_in_filter)))
        error_rate = float(false_pos) / len(self._not_in_filter)
        self.assertTrue(error_rate < 100 * self.FILTER_ERROR_RATE,
                        '%r / %r = %r > %r' % (false_pos,
                                               len(self._not_in_filter),
                                               error_rate,
                                               100 * self.FILTER_ERROR_RATE))
        for item in self._not_in_filter:
            # We should *never* have a false negative
            self.assertFalse(item in bf, '%r WAS in %r' % (item, bf))

    def test_repr(self):
        self.assertEqual(
            '<BloomFilter capacity: %d, error: %0.3f, num_hashes: %d>' % (
                self.bf.capacity, self.bf.error_rate, self.bf.num_hashes),
            repr(self.bf))
        self.assertEqual(
            '<BloomFilter capacity: %d, error: %0.3f, num_hashes: %d>' % (
                self.bf.capacity, self.bf.error_rate, self.bf.num_hashes),
            str(self.bf))
        self.assertEqual(
            '<BloomFilter capacity: %d, error: %0.3f, num_hashes: %d>' % (
                self.bf.capacity, self.bf.error_rate, self.bf.num_hashes),
            str(self.bf))

    def test_filename(self):
        # .name is pending deprecation, ensure .filename is equivalent
        self.assertEqual(self.bf.name.decode(), self.bf.filename)

    def test_add_and_check_file_backed(self):
        self._populate_filter(self.bf)
        self._check_filter_contents(self.bf)

    def test_update_and_check_file_backed(self):
        self._populate_filter(self.bf, use_update=True)
        self._check_filter_contents(self.bf)

    def test_add_and_check_memory_backed(self):
        self._populate_filter(self.bf_mem)
        self._check_filter_contents(self.bf_mem)

    def test_open(self):
        self._populate_filter(self.bf)
        self.assertEqual(self.bf.read_only, False)
        self.bf.sync()

        # Read and write
        bf1 = pybloomfilter.BloomFilter.open(self.bf.filename)
        self._check_filter_contents(bf1)
        self.assertEqual(bf1.read_only, False)

        bf2 = pybloomfilter.BloomFilter.open(self.bf.filename, mode="rw")
        self._check_filter_contents(bf2)
        self.assertEqual(bf2.read_only, False)

        # Read only
        bfro = pybloomfilter.BloomFilter.open(self.bf.filename, mode="r")
        self._check_filter_contents(bfro)
        self.assertEqual(bfro.read_only, True)

    def test_open_missing_file_is_os_error(self):
        self.assertRaises(OSError, pybloomfilter.BloomFilter.open,
                            "missing_directory/some_file.bloom", "r")
        self.assertRaises(OSError, pybloomfilter.BloomFilter.open,
                            "missing_directory/some_file.bloom", "rw")

    def test_read_only_write_is_value_error(self):
        bfro = pybloomfilter.BloomFilter.open(self.tempfile.name, mode="r")
        self.assertEqual(bfro.read_only, True)
        self.assertRaises(ValueError, bfro.add, "test")
        self.assertRaises(ValueError, bfro.update, ["test"])
        self.assertRaises(ValueError, bfro.sync)
        self.assertRaises(ValueError, bfro.clear_all)

    def test_read_only_set_operations_is_value_error(self):
        bf_mem = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                           self.FILTER_ERROR_RATE)

        bfro = pybloomfilter.BloomFilter.open(self.tempfile.name, mode="r")
        self.assertEqual(bfro.read_only, True)
        self.assertRaises(ValueError, bfro.union, bf_mem)
        self.assertRaises(ValueError, bfro.intersection, bf_mem)
        self.assertRaises(ValueError, bfro.__ior__, bf_mem)
        self.assertRaises(ValueError, bfro.__iand__, bf_mem)

    @with_test_file
    def test_copy(self, filename):
        self._populate_filter(self.bf)
        self.bf.sync()

        bf = self.bf.copy(filename)
        self._check_filter_contents(bf)
        self.assertPropertiesPreserved(self.bf, bf)
        self.assertEqual(bf.read_only, False)

    def assertBfPermissions(self, bf, perms):
        oct_mode = oct(os.stat(bf.filename).st_mode)
        self.assertTrue(oct_mode.endswith(perms),
                     'unexpected perms %s' % oct_mode)

    @with_test_file
    def test_to_from_base64(self, filename):
        self._populate_filter(self.bf)
        self.bf.sync()

        # sanity-check
        self.assertBfPermissions(self.bf, '0755')

        b64 = self.bf.to_base64()

        old_umask = os.umask(0)
        try:
            os.unlink(filename)
            bf = pybloomfilter.BloomFilter.from_base64(filename, b64,
                                                       perm=0o775)
            self.assertBfPermissions(bf, '0775')
            self._check_filter_contents(bf)
            self.assertPropertiesPreserved(self.bf, bf)
        finally:
            os.umask(old_umask)

    def test_missing_file_is_os_error(self):
        self.assertRaises(OSError, pybloomfilter.BloomFilter, 1000, 0.1,
                          'missing_directory/some_file.bloom')

    @with_test_file
    def test_others(self, filename):
        bf = pybloomfilter.BloomFilter(100, 0.01, filename)
        for elem in (1.2, 2343, (1, 2), object(), '\u2131\u3184'):
            bf.add(elem)
            self.assertEqual(elem in bf, True)

    def test_number_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        bf.add(1234)
        self.assertEqual(1234 in bf, True)

    def test_string_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        bf.add("test")
        self.assertEqual("test" in bf, True)

    def test_others_nofile(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        for elem in (1.2, 2343, (1, 2), object(), '\u2131\u3184'):
            bf.add(elem)
            self.assertEqual(elem in bf, True)

    @with_test_file
    def _test_large_file(self, filename):
        bf = pybloomfilter.BloomFilter(400000000, 0.01, filename)
        bf.add(1234)
        self.assertEqual(1234 in bf, True)

    def test_name_does_not_segfault(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        self.assertRaises(NotImplementedError, lambda: bf.filename)

    def test_copy_does_not_segfault(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        with tempfile.NamedTemporaryFile(suffix='.bloom') as f2:
            self.assertRaises(NotImplementedError, bf.copy, f2.name)

    def test_to_base64_does_not_segfault(self):
        bf = pybloomfilter.BloomFilter(100, 0.01)
        self.assertRaises(NotImplementedError, bf.to_base64)

    def test_copy_template(self):
        self._populate_filter(self.bf)
        with tempfile.NamedTemporaryFile() as _file:
            bf2 = self.bf.copy_template(_file.name)
            self.assertPropertiesPreserved(self.bf, bf2)
            self.assertEqual(bf2.read_only, False)
            bf2.union(self.bf)  # Asserts copied bloom filter is comparable
            self._check_filter_contents(bf2)

    def test_create_with_hash_seeds(self):
        cust_seeds = [getrandbits(32) for i in range(30)]
        bf = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                       self.FILTER_ERROR_RATE,
                                       self.tempfile.name,
                                       hash_seeds=cust_seeds)
        bf_seeds = bf.hash_seeds.tolist()
        self.assertEqual(cust_seeds, bf_seeds)

    def test_create_with_hash_seeds_invalid(self):
        cust_seeds = ["ABC", -123, "123456", getrandbits(33)]
        self.assertRaises(ValueError,
                          pybloomfilter.BloomFilter,
                          self.FILTER_SIZE,
                          self.FILTER_ERROR_RATE,
                          hash_seeds=cust_seeds)

    def test_create_with_hash_seeds_and_compare(self):
        test_data = "test"
        bf1 = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                        self.FILTER_ERROR_RATE,
                                        self.tempfile.name)
        bf1.add(test_data)
        bf1_seeds = bf1.hash_seeds.tolist()
        bf1_ba = bf1.bit_array

        bf2 = pybloomfilter.BloomFilter(self.FILTER_SIZE,
                                        self.FILTER_ERROR_RATE,
                                        self.tempfile.name,
                                        hash_seeds=bf1_seeds)
        bf2.add(test_data)
        bf2_seeds = bf2.hash_seeds.tolist()
        bf2_ba = bf2.bit_array

        self.assertEqual(bf1_seeds, bf2_seeds)

        # Expecting same hashing sequence
        self.assertEqual(bf1_ba, bf2_ba)

    def test_bit_array(self):
        bf = pybloomfilter.BloomFilter(1000, 0.01, self.tempfile.name)
        bf.add("apple")

        # Count the number of 1s
        total_ones = 0
        bit_array_str = bin(bf.bit_array)
        for c in bit_array_str:
            if c == "1":
                total_ones += 1

        # For the first item addition, BF should contain
        # the same amount of 1s as the number of hashes
        # performed
        assert total_ones == bf.num_hashes

        for i in range(1000):
            bf.add(randint(0, 1000))

        bf.add("apple")
        ba_1 = bf.bit_array

        bf.add("apple")
        ba_2 = bf.bit_array

        # Should be the same
        assert ba_1 ^ ba_2 == 0

        bf.add("pear")
        bf.add("mango")
        ba_3 = bf.bit_array

        # Should not be the same
        assert ba_1 ^ ba_3 != 0

    def test_bit_array_same_hashes(self):
        capacity = 100 * 100
        items = []
        for i in range(capacity):
            items.append(randint(0, 1000))
        
        # File-backed
        bf1 = pybloomfilter.BloomFilter(capacity, 0.01, self.tempfile.name)
        bf1.update(items)

        bf1_hs = bf1.hash_seeds
        bf1_ba = bf1.bit_array

        # In-memory
        bf2 = pybloomfilter.BloomFilter(capacity, 0.01, hash_seeds=bf1_hs)
        bf2.update(items)

        bf2_ba = bf2.bit_array

        # Should be identical as data was hashed into the same locations
        assert bf1_ba ^ bf2_ba == 0


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SimpleTestCase))
    return suite
